from django.urls import path

from ads.views import AdImpressionView, AdClickView


app_name = 'ads'
urlpatterns = [
    path('<int:pk>/', AdClickView.as_view(), name='ad-click'),
    path('get-ads-by-zones/', AdImpressionView.as_view(), name='ad-impression'),
]



from __future__ import unicode_literals

from django import template
from django.conf import settings
from django.utils import timezone

from ads.models import Ad

register = template.Library()


@register.inclusion_tag('ads/tags/render_ads_zone.html', takes_context=True)
def render_ads_zone(context, zone):
    """
    Returns an advertise for a ``zone``.
    Tag usage:
    {% load ads_tags %}
    {% render_zone 'zone' %}
    """
    context.update({
        'google_adsense_client': settings.ADS_GOOGLE_ADSENSE_CLIENT,
        'zone': zone,
        'zone_info': settings.ADS_ZONES.get(zone, None)
    })
    return context


@register.simple_tag
def get_ads_count(zone):
    """
    Returns ads count for ``zone``.
    Tag usage:
    {% load ads_tags %}
    {% get_ads_count 'zone' as ads_count %}
    {% get_ads_count 'zone1,zone2,zone3' as ads_count %}
    """
    zone = zone.split(',')
    return Ad.objects.public().filter(zone__in=zone).count()



from __future__ import unicode_literals

import csv

from django import forms
from django.contrib import admin
from django.http import HttpResponse

from ads.forms import AdImageInlineForm
from ads.models import *
from ads.utils import get_zones_choices


class AdvertiserAdmin(admin.ModelAdmin):
    search_fields = ['company_name', 'website']
    list_display = ['company_name', 'website', 'created_by']
    raw_id_fields = ['created_by']

    def get_changeform_initial_data(self, request):
        """
        Provide initial datas when creating an advertiser.
        """
        get_data = super(AdvertiserAdmin, self).get_changeform_initial_data(request)
        return get_data or {
            'created_by': request.user.pk
        }


class CategoryAdmin(admin.ModelAdmin):
    list_display = ['title', 'created_by']
    raw_id_fields = ['created_by']

    def get_changeform_initial_data(self, request):
        """
        Provide initial datas when creating a category.
        """
        get_data = super(CategoryAdmin, self). \
            get_changeform_initial_data(request)
        return get_data or {
            'created_by': request.user.pk
        }


class AdAdminForm(forms.ModelForm):
    class Meta:
        fields = '__all__'
        widgets = {
            'zone': forms.Select(choices=get_zones_choices())
        }


class AdImageInline(admin.TabularInline):
    model = AdImage
    form = AdImageInlineForm
    fields = ('device', 'image', )


class AdAdmin(admin.ModelAdmin):
    form = AdAdminForm
    list_display = [
        'title', 'url', 'zone', 'advertiser', 'weight',
        'publication_date', 'publication_date_end']
    list_filter = [
        'publication_date', 'publication_date_end',
        'created_at', 'modified_at']
    search_fields = ['title', 'url']
    raw_id_fields = ['advertiser', 'created_by']
    inlines = (AdImageInline, )

    def get_changeform_initial_data(self, request):
        """
        Provide initial datas when creating an Ad.
        """
        get_data = super(AdAdmin, self).get_changeform_initial_data(request)
        return get_data or {
            'created_by': request.user.pk
        }


class ClickAdmin(admin.ModelAdmin):
    search_fields = ['ad__title', 'source_ip', 'session_id']
    list_display = ['ad', 'click_date', 'source_ip', 'session_id']
    list_filter = ['ad', 'click_date', 'ad__zone']
    date_hierarchy = 'click_date'
    actions = ['download_clicks']

    def get_queryset(self, request):
        qs = super(ClickAdmin, self).get_queryset(request)
        return qs.select_related('ad', 'ad__advertiser')

    def download_clicks(self, request, queryset):
        response = HttpResponse(content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename="clicks.csv"'
        writer = csv.writer(response)
        writer.writerow(('Title',
                         'Advertised URL',
                         'Source IP',
                         'Timestamp',
                         'Advertiser ID',
                         'Advertiser name',
                         'Zone'))
        for click in queryset:
            writer.writerow((unicode(click.ad.title).encode("utf-8"),
                             click.ad.url,
                             click.source_ip,
                             click.click_date.isoformat(),
                             click.ad.advertiser.pk,
                             unicode(click.ad.advertiser.company_name).encode("utf-8"),
                             unicode(click.ad.zone).encode("utf-8")))
        return response
    download_clicks.short_description = "Download selected Ad Clicks"


class ImpressionAdmin(admin.ModelAdmin):
    search_fields = ['ad__title', 'source_ip', 'session_id']
    list_display = ['ad', 'impression_date', 'source_ip', 'session_id']
    list_filter = ['ad', 'impression_date', 'ad__zone']
    date_hierarchy = 'impression_date'
    actions = ['download_impressions']

    def get_queryset(self, request):
        qs = super(ImpressionAdmin, self).get_queryset(request)
        return qs.select_related('ad', 'ad__advertiser')

    def download_impressions(self, request, queryset):
        response = HttpResponse(content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename="impressions.csv"'
        writer = csv.writer(response)
        writer.writerow(('Title',
                         'Advertised URL',
                         'Source IP',
                         'Timestamp',
                         'Advertiser ID',
                         'Advertiser name',
                         'Zone'))
        for impression in queryset:
            writer.writerow((unicode(impression.ad.title).encode("utf-8"),
                             impression.ad.url,
                             impression.source_ip,
                             impression.impression_date.isoformat(),
                             impression.ad.advertiser.pk,
                             unicode(impression.ad.advertiser.company_name).encode("utf-8"),
                             unicode(impression.ad.zone).encode("utf-8")))
        return response
    download_impressions.short_description = "Download selected Ad Impressions"


admin.site.register(Advertiser, AdvertiserAdmin)
admin.site.register(Category, CategoryAdmin)
admin.site.register(Ad, AdAdmin)
admin.site.register(Click, ClickAdmin)
admin.site.register(Impression, ImpressionAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-02-07 08:34
from __future__ import unicode_literals

from django.db import migrations


def forwards_func(apps, schema_editor):
    # We get the model from the versioned app registry;
    # if we directly import it, it'll be the wrong version
    Ad = apps.get_model("ads", "Ad")
    AdImage = apps.get_model("ads", "AdImage")
    for ad in Ad.objects.all():
        AdImage.objects.create(
            device='sm',
            image=ad.image)


def reverse_func(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('ads', '0002_adimage'),
    ]

    operations = [
        migrations.RunPython(forwards_func, reverse_func),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2016-12-19 22:26
from __future__ import unicode_literals

import ads.models
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Ad',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('image', models.ImageField(max_length=255, upload_to=b'', verbose_name='Image')),
                ('url', models.URLField(verbose_name='Advertised URL')),
                ('publication_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Start showing')),
                ('publication_date_end', models.DateTimeField(default=ads.models.now_plus_1_day, verbose_name='Stop showing')),
                ('zone', models.CharField(max_length=100, verbose_name='Zone')),
                ('weight', models.IntegerField(default=1, help_text='Weight of the ad relative to other ads in the same zone.<br />Ad with higher weight will be displayed more frequently.', validators=[django.core.validators.MinValueValidator(1)], verbose_name='Weight')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'verbose_name': 'Ad',
                'verbose_name_plural': 'Ads',
            },
        ),
        migrations.CreateModel(
            name='Advertiser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('company_name', models.CharField(max_length=255, verbose_name='Company Name')),
                ('website', models.URLField(verbose_name='Company Site')),
                ('created_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Created By')),
            ],
            options={
                'ordering': ('company_name',),
                'verbose_name': 'Advertiser',
                'verbose_name_plural': 'Advertisers',
            },
        ),
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('description', models.TextField(blank=True, verbose_name='Description')),
                ('created_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Created By')),
            ],
            options={
                'ordering': ('title',),
                'verbose_name': 'Category',
                'verbose_name_plural': 'Categories',
            },
        ),
        migrations.CreateModel(
            name='Click',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('click_date', models.DateTimeField(auto_now_add=True, verbose_name='When')),
                ('source_ip', models.GenericIPAddressField(blank=True, null=True, verbose_name='Source IP Address')),
                ('session_id', models.CharField(blank=True, max_length=40, null=True, verbose_name='Source Session ID')),
                ('ad', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='clicks', to='ads.Ad', verbose_name='Ad')),
            ],
            options={
                'verbose_name': 'Ad Click',
                'verbose_name_plural': 'Ad Clicks',
            },
        ),
        migrations.CreateModel(
            name='Impression',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('impression_date', models.DateTimeField(auto_now_add=True, verbose_name='When')),
                ('source_ip', models.GenericIPAddressField(blank=True, null=True, verbose_name='Source IP Address')),
                ('session_id', models.CharField(blank=True, max_length=40, null=True, verbose_name='Source Session ID')),
                ('ad', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='impressions', to='ads.Ad', verbose_name='Ad')),
            ],
            options={
                'verbose_name': 'Ad Impression',
                'verbose_name_plural': 'Ad Impressions',
            },
        ),
        migrations.AddField(
            model_name='ad',
            name='advertiser',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ads.Advertiser', verbose_name='Ad Provider'),
        ),
        migrations.AddField(
            model_name='ad',
            name='category',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='ads.Category', verbose_name='Category'),
        ),
        migrations.AddField(
            model_name='ad',
            name='created_by',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Created By'),
        ),
        migrations.AlterIndexTogether(
            name='impression',
            index_together=set([('ad', 'session_id')]),
        ),
        migrations.AlterIndexTogether(
            name='click',
            index_together=set([('ad', 'session_id')]),
        ),
    ]



from django.conf import settings
from appconf import AppConf

from ads.compat import gettext_lazy as _


gettext = lambda s: s


class AdsConf(AppConf):

    class Meta:
        prefix = 'ads'

    GOOGLE_ADSENSE_CLIENT = None  # 'ca-pub-xxxxxxxxxxxxxxxx'

    ZONES = {
        'header': {
            'name': gettext('Header'),
            'ad_size': {
                'xs': '720x150',
                'sm': '800x90',
                'md': '800x90',
                'lg': '800x90',
                'xl': '800x90'                
            },
            'google_adsense_slot': None,  # 'xxxxxxxxx',
            'google_adsense_format': None,  # 'auto'
        },
        'content': {
            'name': gettext('Content'),
            'ad_size': {
                'xs': '720x150',
                'sm': '800x90',
                'md': '800x90',
                'lg': '800x90',
                'xl': '800x90'                
            },
            'google_adsense_slot': None,  # 'xxxxxxxxx',
            'google_adsense_format': None,  # 'auto'
        },
        'sidebar': {
            'name': gettext('Sidebar'),
            'ad_size': {
                'xs': '720x150',
                'sm': '800x90',
                'md': '800x90',
                'lg': '800x90',
                'xl': '800x90'
            }
        }
    }

    DEFAULT_AD_SIZE = '720x150'

    DEVICES = (
        ('xs', _('Extra small devices')),
        ('sm', _('Small devices')),
        ('md', _('Medium devices (Tablets)')),
        ('lg', _('Large devices (Desktops)')),
        ('xl', _('Extra large devices (Large Desktops)')),
    )
    
    VIEWPORTS = {
        'xs': 'd-block img-fluid d-sm-none',
        'sm': 'd-none img-fluid d-sm-block d-md-none',
        'md': 'd-none img-fluid d-md-block d-lg-none',
        'lg': 'd-none img-fluid d-lg-block d-xl-none',
        'xl': 'd-none img-fluid d-xl-block',
    }



import random

from ads.querysets import AdQuerySet
from django.db.models import Manager, Count


class AdManager(Manager):
    def get_queryset(self):
        return AdQuerySet(self.model)

    def public(self):
        return self.get_queryset().public()

    def zone_ads(self, zone):
        return self.get_queryset().zone_ads(zone)

    def random_ad(self, zone):
        ads = []
        for ad in self.zone_ads(zone).public():
            ads += [ad] * ad.weight
        if not ads:
            return None
        return random.choice(ads)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-02-08 10:15
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ads', '0003_auto_20170207_1034'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='ad',
            name='image',
        ),
    ]



from django.db.models import QuerySet
from django.utils import timezone


class AdQuerySet(QuerySet):

    def public(self):
        return self.filter(
            publication_date__lte=timezone.now(),
            publication_date_end__gt=timezone.now())

    def zone_ads(self, zone):
        return self.filter(
            zone=zone)



from django.conf import settings
from django.http import HttpResponseRedirect
from django.utils import timezone
from django.views import View
from django.views.generic.detail import SingleObjectMixin

from braces.views import JSONResponseMixin

from ads.models import Ad, Click
from ads.utils import update_clicks, update_impressions


class AdImpressionView(JSONResponseMixin, View):
    json_dumps_kwargs = {u"indent": 2}

    def get_object(self):
        zone = self.kwargs.get('zone', None)
        return Ad.objects.random_ad(zone)

    def get_ad_context_dict(self, zone):
        ad = Ad.objects.random_ad(zone)
        if ad:
            context_dict = {
                'url': ad.get_absolute_url(),
                'images': {}
            }
            for image in ad.images.all():
                context_dict['images'].update({
                    image.device: {
                        'url': image.image.url,
                        'size': image.size
                    }
                })
            return context_dict
        return None
        
    def get(self, request, *args, **kwargs):
        data = {}
        zones = request.GET.getlist('zones[]', []);
        # print(zones)
        for zone in zones:
            zone_conf = settings.ADS_ZONES.get(zone, {})
            ad = self.get_ad_context_dict(zone)
            if zone_conf:
                data.update({
                    zone: {
                        'ad': ad,
                        'conf': zone_conf
                    }
                })
        context_dict = {
            'google_adsense_client': settings.ADS_GOOGLE_ADSENSE_CLIENT,
            'viewports': settings.ADS_VIEWPORTS,
            'zones': data
        }
        return self.render_json_response(context_dict)
    
        """
        context_dict = {
            'zone': self.kwargs.get('zone', None)
        }
        ad = self.get_object()
        if ad:
            ct = context_dict.update({
                'url': ad.url,
                'images': {}
            })
            for image in ad.images.all():
                context_dict['images'].update({
                    image.device: {
                        'image': image.image.url,
                        'size': image.size
                    }
                })
        return self.render_json_response(context_dict)
        """

class AdClickView(SingleObjectMixin, View):

    def get_queryset(self):
        return Ad.objects.all()

    def get(self, request, *args, **kwargs):
        ad = self.get_object()
        update_clicks(ad, request)
        return HttpResponseRedirect(ad.url)



"""Compatibility helpers for supporting multiple Django versions."""
import django

if django.VERSION < (4, 0):
    from django.utils.translation import ugettext_lazy as gettext_lazy  # type: ignore
    from django.utils.translation import ugettext as gettext  # type: ignore
else:
    from django.utils.translation import gettext_lazy  # type: ignore
    from django.utils.translation import gettext  # type: ignore

try:  # pragma: no cover - depends on Django version
    from django.utils.encoding import force_str
except ImportError:  # Django < 3.0
    from django.utils.encoding import force_text as force_str  # type: ignore

__all__ = ["gettext", "gettext_lazy", "force_str"]



from django.conf import settings
from django.utils import timezone

from ads.compat import gettext_lazy as _

from ads.models import Click, Impression


def get_zones_choices():
    for key in sorted(settings.ADS_ZONES):
        yield (key, _(settings.ADS_ZONES[key].get('name', 'Undefined')))


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR', None)
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR', '')
    return ip


def update_clicks(ad, request):
    if ad is not None:
        if request.session.session_key:
            impression, created = Click.objects.get_or_create(
                ad=ad,
                session_id=request.session.session_key,
                defaults={
                    'click_date': timezone.now(),
                    'source_ip': get_client_ip(request),
                })


def update_impressions(ad, request):
    if ad is not None:
        if request.session.session_key:
            impression, created = Impression.objects.get_or_create(
                ad=ad,
                session_id=request.session.session_key,
                defaults={
                    'impression_date': timezone.now(),
                    'source_ip': get_client_ip(request),
                })



from django.http import JsonResponse


class JSONResponseMixin:
    """
    A mixin that can be used to render a JSON response.
    """
    def render_to_json_response(self, context, **response_kwargs):
        """
        Returns a JSON response, transforming 'context' to make the payload.
        """
        return JsonResponse(
            self.get_data(context),
            **response_kwargs
        )

    def get_data(self, context):
        """
        Returns an object that will be serialized as JSON by json.dumps().
        """
        # Note: This is *EXTREMELY* naive; in reality, you'll need
        # to do much more complex handling to ensure that arbitrary
        # objects -- such as Django model instances or querysets
        # -- can be serialized as JSON.
        return context



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-26 10:45
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ads', '0004_remove_ad_image'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ad',
            name='created_by',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='Created By'),
        ),
        migrations.AlterField(
            model_name='adimage',
            name='device',
            field=models.CharField(choices=[('xs', 'Smartphones'), ('sm', 'Tablets'), ('md', 'Small Desktops'), ('lg', 'Large Desktops')], max_length=2, verbose_name='Device'),
        ),
        migrations.AlterField(
            model_name='adimage',
            name='image',
            field=models.ImageField(max_length=255, upload_to='', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='advertiser',
            name='created_by',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='Created By'),
        ),
        migrations.AlterField(
            model_name='category',
            name='created_by',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='Created By'),
        ),
    ]



from __future__ import unicode_literals

from django.apps import AppConfig

from ads.compat import gettext_lazy as _


class AdsConfig(AppConfig):
    name = 'ads'
    default_auto_field = 'django.db.models.BigAutoField'
    verbose_name = _('Ads Management System')



from __future__ import unicode_literals

from django.core.validators import MinValueValidator
from django.db import models
from django.urls import reverse
from django.utils import timezone

from ads.compat import force_str, gettext_lazy as _

from ads.conf import settings
from ads.managers import AdManager


class Advertiser(models.Model):
    """ A Model for our Advertiser.  """
    company_name = models.CharField(
        verbose_name=_(u'Company Name'), max_length=255)
    website = models.URLField(verbose_name=_(u'Company Site'))
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, null=True, on_delete=models.SET_NULL,
        verbose_name=_('Created By'))

    class Meta:
        verbose_name = _('Advertiser')
        verbose_name_plural = _('Advertisers')
        ordering = ('company_name',)

    def __str__(self):
        return self.company_name

    def get_website_url(self):
        return self.website


class Category(models.Model):
    """ a Model to hold the different Categories for adverts """
    title = models.CharField(
        verbose_name=_('Title'), max_length=255)
    description = models.TextField(
        verbose_name=_('Description'), blank=True)
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, null=True, on_delete=models.SET_NULL,
        verbose_name=_('Created By'))

    class Meta:
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')
        ordering = ('title',)

    def __str__(self):
        return self.title


def now_plus_1_day():
    return timezone.now() + timezone.timedelta(days=1)


class Ad(models.Model):
    """
    This is our base model, from which all ads will inherit.
    The manager methods for this model will determine which ads to
    display return etc.
    """
    title = models.CharField(verbose_name=_('Title'), max_length=255)
    url = models.URLField(verbose_name=_('Advertised URL'))

    publication_date = models.DateTimeField(
        verbose_name=_('Start showing'),
        default=timezone.now)
    publication_date_end = models.DateTimeField(
        verbose_name=_('Stop showing'),
        default=now_plus_1_day)

    # Relations
    advertiser = models.ForeignKey(
        Advertiser, on_delete=models.CASCADE, verbose_name=_("Ad Provider"))
    category = models.ForeignKey(
        Category, on_delete=models.CASCADE,
        verbose_name=_("Category"), blank=True, null=True)
    zone = models.CharField(
        verbose_name=_('Zone'), max_length=100)
    weight = models.IntegerField(
        verbose_name=_('Weight'),
        help_text=_('Weight of the ad relative to other ads '
                    'in the same zone.<br />'
                    'Ad with higher weight will be '
                    'displayed more frequently.'),
        default=1,
        validators=[MinValueValidator(1)])

    created_at = models.DateTimeField(auto_now_add=True)
    modified_at = models.DateTimeField(auto_now=True)
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, null=True, on_delete=models.SET_NULL,
        verbose_name=_('Created By'))

    objects = AdManager()

    class Meta:
        verbose_name = _('Ad')
        verbose_name_plural = _('Ads')

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('ads:ad-click', kwargs={
            'pk': self.id})


class AdImage(models.Model):
    ad = models.ForeignKey(
        Ad, on_delete=models.CASCADE, verbose_name=_('Ad'),
        related_name='images')
    device = models.CharField(
        verbose_name=_('Device'), max_length=2, choices=settings.ADS_DEVICES)
    image = models.ImageField(verbose_name=_('Image'), max_length=255)

    @property
    def size(self):
        size = settings.ADS_ZONES.get(self.ad.zone, {}). \
            get('ad_size', {}). \
            get(self.device, None)
        return size or settings.ADS_DEFAULT_AD_SIZE

    def __str__(self):
        return self.get_device_display()


class Impression(models.Model):
    """
    The AdImpression Model will record every time the ad is loaded on a page
    """
    ad = models.ForeignKey(
        Ad, on_delete=models.CASCADE, verbose_name=_('Ad'),
        related_name='impressions')
    impression_date = models.DateTimeField(
        verbose_name=_('When'), auto_now_add=True)
    source_ip = models.GenericIPAddressField(
        verbose_name=_('Source IP Address'), null=True, blank=True)
    session_id = models.CharField(
        verbose_name=_('Source Session ID'),
        max_length=40, null=True, blank=True)

    class Meta:
        verbose_name = _('Ad Impression')
        verbose_name_plural = _('Ad Impressions')
        index_together = (
            ('ad', 'session_id', )
        )

    def __str__(self):
        return force_str(self.ad)


class Click(models.Model):
    """
    The AdClick model will record every click that a add gets
    """
    ad = models.ForeignKey(
        Ad, on_delete=models.CASCADE, verbose_name=_('Ad'),
        related_name='clicks')
    click_date = models.DateTimeField(
        verbose_name=_('When'), auto_now_add=True)
    source_ip = models.GenericIPAddressField(
        verbose_name=_('Source IP Address'), null=True, blank=True)
    session_id = models.CharField(
        verbose_name=_('Source Session ID'),
        max_length=40, null=True, blank=True)

    class Meta:
        verbose_name = _('Ad Click')
        verbose_name_plural = _('Ad Clicks')
        index_together = (
            ('ad', 'session_id', )
        )

    def __str__(self):
        return force_str(self.ad)
